/*
 * Useful macroses.
 *
 * Grisha Sitkarev, <sitkarev@unixkomi.ru>
 *
 */

#ifndef MACROS_H_
#define MACROS_H_

#ifndef FALSE
#define FALSE	0
#endif
#ifndef TRUE
#define TRUE	!(FALSE)
#endif

#define N_ELEMS(a)	(sizeof(a)/sizeof((a)[0]))

#define OBJ_NEW(obj) \
do { \
	obj = malloc(sizeof(*obj)); \
	if (obj == NULL) { \
		fprintf(stderr, "out of memory\n"); \
		exit(2); \
	} \
	memset(obj, 0, sizeof(*obj)); \
} while (0);

#define FREE(obj) \
do { \
	free(obj); \
	obj = NULL; \
} while (0)

#ifdef DEBUG
#define debug(fmt, arg...)	do { fprintf(stderr, fmt "\n", ##arg); } while (0) 
#else
#define debug(fmt, arg...)	do { } while (0)
#endif

#define error(fmt, arg...)	do { fprintf(stderr, fmt "\n", ##arg); exit(1); } while (0)

#ifdef DEBUG
#define return_if_fail(expr) \
do { \
  if (!(expr)) { \
    fprintf(stderr, "%s:%d:%s(): expression `%s' failed!\n", \
	__FILE__, \
	__LINE__, \
	__FUNCTION__, \
	#expr); \
    return; \
  } \
} while (0)	

#define return_val_if_fail(expr, val) \
do { \
  if (!(expr)) { \
    fprintf(stderr, "%s:%d:%s(): expression `%s' failed!\n", \
        __FILE__, \
        __LINE__, \
        __FUNCTION__, \
        #expr); \
    return (val); \
  } \
} while (0)

#define TRACE(msg, arg...) \
do { \
  fprintf(stderr, "%s:%d:%s(): " msg "\n", \
      __FILE__, __LINE__, __FUNCTION__, ##arg); \
} while (0)
#else
#define return_if_fail(expr)
#define return_val_if_fail(expr, val)
#define TRACE(msg, arg...)
#endif

#endif /* MACROS_H_ */
